#!/usr/bin/env python

VERSION = "4.0.2"

# New arguments strategy: oe [bakery options] <command> [command options]
#
# the bakery options should only apply to the bakery tool itself, and not the
# command as such (even if a builtin command).  Only bakery options supported
# are "-d|--debug" for bakery tool debugging, "-v|--version" for bakery tool
# version information, "-h|--help" for general bakery tool usage information.

import sys
import os
import optparse
import logging

# Force stdout and stderr into un-buffered mode.  This seems to unbreak the
# FUBAR stdout/stderr handling in Jenkins for now...
sys.stdout = os.fdopen(sys.stdout.fileno(), "w", 0)
sys.stderr = os.fdopen(sys.stderr.fileno(), "w", 0)

# Parse bakery command-line options
parser = optparse.OptionParser(
    """
  %prog [--version] [-h|--help] [-d|--debug] <command> [options] [<args>]""",
    version="OE-lite Bakery %s"%(VERSION))
parser.add_option("-d", "--debug",
                  action="store_true",
                  help="Debug the OE-lite Bakery tool itself")
parser.disable_interspersed_args()
bakery_options, cmd_argv = parser.parse_args(sys.argv[1:])

def module_version(module):
    try:
        return module.__version__
    except AttributeError:
        return "unknown"

# Import oebakery modules
try:
    import oebakery
    if module_version(oebakery) != VERSION:
        del sys.modules["oebakery"]
        del oebakery
        raise Exception()
except:
    if bakery_options.debug:
        print "DEBUG: bakery: importing oebakery module from source directory"
    sys.path.insert(
        0, os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
    try:
        import oebakery
        version = module_version(oebakery)
        if version != VERSION:
            print >>sys.stderr, "CRITICAL: bad oebakery module version: %s (expected: %s)"%(version, VERSION)
            sys.exit(1)
    except ImportError, e:
        print >>sys.stderr, "CRITICAL: cannot import oebakery module"
        sys.exit(1)

# Initialize logging
logger = oebakery.logger
if bakery_options.debug:
    logger.setLevel(logging.DEBUG)
else:
    logger.setLevel(logging.INFO)

logger.debug("OE-lite Bakery %s", oebakery.__version__)
logger.debug("bakery_options = %s", bakery_options)
logger.debug("cmd_argv = %r", cmd_argv)

# Show list of available commands
if not cmd_argv or (len(cmd_argv) == 1 and cmd_argv[0] == "help"):
    config = oebakery.parse.parse_bakery_conf()
    oebakery.cmd.add_builtin_cmds()
    oebakery.cmd.add_manifest_cmds()
    usage = parser.get_usage() + "\n" + oebakery.cmd.cmds_usage()
    print usage
    sys.exit(1)

if cmd_argv[0] == "help":
    cmd_argv = cmd_argv[1:] + ["-h"]

config = None

while cmd_argv:
    logger.debug(" ".join(cmd_argv))
    cmd_name = cmd_argv.pop(0)
    oebakery.cmd.clear()
    oebakery.cmd.add_builtin_cmds()
    cmd = oebakery.cmd.get_cmd(cmd_name)

    if config is None and (cmd is None or not "no-bakery-conf" in cmd.flags):
        config = oebakery.parse.parse_bakery_conf()

    if cmd is None:
        oebakery.cmd.add_manifest_cmds()
        cmd = oebakery.cmd.get_cmd(cmd_name)

    if cmd is None:
        logger.critical("Invalid command: %s", cmd_name)
        logger.info("Use '%s help' to get list of available commands.",
                     os.path.basename(sys.argv[0]))
        sys.exit(1)

    parser = oebakery.cmd.cmd_parser(cmd)
    if parser is None:
        logger.critical("%s failed", cmd.name)
        sys.exit(1)
    cmd_options, cmd_args = parser.parse_args(cmd_argv)

    ret = oebakery.cmd.call(cmd, "parse_args", cmd_options, cmd_args)
    if ret:
        logger.critical("%s failed: %s", cmd.name, ret)
        sys.exit(1)

    ret = oebakery.cmd.call(cmd, "run", cmd_options, cmd_args, config)
    if isinstance(ret, list):
        cmd_argv = ret
        continue
    elif ret:
        logger.critical("%s failed: %s", cmd.name, ret)
        sys.exit(1)
    else:
        cmd_argv = []

logger.debug("all done")
sys.exit(0)



import optparse
import os


import oebakery
logger = oebakery.logger


__cmds = {}


def get_cmd(name):
    __cmds
    if not name in __cmds:
        return None
    return __cmds[name]


def clear():
    __cmds = {}


def add_cmd(module, cmd_name, cmd_type):
    logger.debug("adding %s command: %s", cmd_type, cmd_name)
    cmd = __import__("%s.%s"%(module, cmd_name.replace("-", "_")),
                     globals(), locals(),
                     ["run", "description", "arguments"], -1)
    cmd.name = cmd_name
    if not "flags" in dir(cmd):
        cmd.flags = ()
    __cmds[cmd_name] = cmd
    return


def add_builtin_cmds():
    for cmd_name in oebakery.cmd.builtin_cmds:
        add_cmd("oebakery.cmd", cmd_name, "builtin")
    return



def clean_orphaned_pycs(directory):
    """Remove all .pyc files without a correspondent module"""
    for root, dirs, files in os.walk(directory):
        for f in files:
            if f.endswith(".pyc"):
                pyc = os.path.join(root, f)
                if not os.path.exists(pyc[:-1]):
                    logger.debug("deleting orphaned file %s" % pyc)
                    os.unlink(pyc)
    return


def add_manifest_cmds():
    logger.debug("add_manifest_cmds()")
    try:
        import oelite
        clean_orphaned_pycs(os.path.join(oelite.__path__[0], "cmd"))
        import oelite.cmd
        module = "oelite.cmd"
        cmds = oelite.cmd.manifest_cmds
        cmd_type = "manifest"
    except ImportError, e:
        logger.debug("import oelite.cmd failed", exc_info=True)
        try:
            import oelite.baker
        except:
            return False
        module = "oebakery.cmd"
        cmds = ("bake", "show")
        cmd_type = "legacy manifest"
    for cmd_name in cmds:
        add_cmd(module, cmd_name, cmd_type)
    return True


def cmds_usage():
    usage = "Commands:"
    for cmd_name in sorted(__cmds.keys()):
        cmd = __cmds[cmd_name]
        usage += "\n  %-21s "%(cmd_name)
        if "description" in dir(cmd) and cmd.description:
            if isinstance(cmd.description, basestring):
                usage += cmd.description
            else:
                usage += cmd.description[0]
        else:
            usage += "TBD"
    return usage


def cmd_parser(cmd):
    try:
        description = cmd.description
        if description and not isinstance(description, basestring):
            description = "\n".join(description)
            # FIXME: use something like format_textblock to format to console
            # width and handle the 2-space indentation here instead of
            # requiring command writers to bother about it.
    except AttributeError, e:
        description = ""
    try:
        arguments = ""
        if cmd.arguments:
            description += "\n\nArguments:"
            for (arg_name, arg_descr, arg_opt) in cmd.arguments:
                if arg_opt == 0:
                    arguments += " <%s>"%(arg_name)
                elif arg_opt == 1:
                    arguments += " [%s]"%(arg_name)
                elif arg_opt == 2:
                    arguments += " [%s]*"%(arg_name)
                else:
                    arguments += " ?%s?*"%(arg_name)
                description += "\n  %-21s %s"%(arg_name, arg_descr)
    except AttributeError, e:
        arguments = ""
    parser = optparse.OptionParser("""
  %%prog %s [options]%s

  %s."""%(cmd.name, arguments, description))
    ret = call(cmd, "add_parser_options", parser)
    if ret is not None and ret!= 0:
        logger.error("%s.add_parser_options failed: %r", cmd.name, ret)
        return None
    return parser


def call(cmd, function, *args):
    if not function in dir(cmd):
        return None
    logger.debug("%s.%s()", cmd.name, function)
    function = eval("cmd.%s"%(function))

    try:
        ret = function(*args)
    except Exception, e:
        if hasattr(e, "print_details"):
            e.print_details()
        logger.exception("exception in %s.%s()", cmd.name, function.__name__)
        return "Exception: %s"%(e)

    if not ret:
        return 0
    if isinstance(ret, int):
        return "error: %d"%(ret)
    return ret



import optparse, sys, os
import oebakery


description = "Clone an OE-lite development environment into a new directory"
arguments = (
    ("repository", "OE-lite (git) repository to clone", 0),
    ("directory", "directory to to clone into (default is current dir)", 1))
flags = ("no-bakery-conf")


def parse_args(options, args):
    if not args:
        return "repository argument required"
    if len(args) < 1:
        return "too few arguments"
    if len(args) > 2:
        return "too many arguments"
    options.repository = args.pop(0)
    if args:
        options.directory = args.pop(0)
    else:
        options.directory = options.repository.strip("/")
        options.directory = os.path.basename(options.directory)
        if options.directory[-4:] == '.git':
            options.directory = options.directory[:-4]
    return 0


def run(options, args, config):
    if not oebakery.call('git clone --recursive %s %s'%(
            options.repository, options.directory)):
        return "git clone failed"

    topdir = oebakery.set_topdir(options.directory)
    oebakery.chdir(options.directory)

    oebakery.path.copy_local_conf_sample("conf")

    if not oebakery.call('git config push.default tracking'):
        print 'Failed to set push.default = tracking'

    return ["update"]



import os
import string


import oebakery


description = "Update OE-lite repository according to configuration"
arguments = None

# oe update: (setup submodules based on OETASK in bakery.conf)
#   git submodule sync
#   git submodule add $url $path for missing submodules
#     and: git submodule update --init --recursive $path
#   change bakery repo .gitmodules submodule.$submodule.url if not right
#   change submodule repo .git/config remote.origin.url if not right
#     (or will this automagically be done by next submodule sync???)
#   change submodule repo .git/config remote.origin.pushurl if not right
#   if submodule is tracking a branch:
#     if local $branch does not exist:
#       git branch --track $branch origin/$branch
#     elif submodule repo .git/config branch.$branch.merge != refs/heads/$branch:
#       git branch --set-upstream $branch origin/$branch
#   git submodule sync
#   print warning for submodules not in OETASK
#     (ask to add it to bakery.conf or remove it)
#
#

# oe pull:
#   git pull in bakery repo
#   oe init
#   foreach module in OESTACK:
#     check submodule status
#     if submodule is missing:
#       print error message, and skip submodule
#     if HEAD is not in a local or remote branch:
#       print error message, and skip submodule
#     if submodule is tracking a branch: (ie. ;branch=something in OESTACK)
#       if $branch is not checked out:
#         git checkout $branch
#       git pull in the submodule
#     else:
#       git submodule update --recursive
#


def run(options, args, config):
    if not os.path.exists('.git'):
        return "Aiee! This is not even a git repository: %s"%(os.getcwd())

    # check for possibly detached heads, and abort (return False)
    # if this or at least one of the submodules looks detached

    paths = []
    submodules = []
    for path, url, params in config["__submodules"]:
        if not args or path in args:
            paths.append(path)
            submodules.append((path, url, params))

    return not update_submodules(submodules)


def update_submodules(submodules):
    ok = True
    for path, url, params in submodules:
        if not check_submodule(path):
            logger.error("submodule %s is not on a branch", path)
            ok = False
            continue
        if not update_submodule(path, url, params):
            logger.error("update of submodule %s failed", path)
            ok = False

    if not ok:
        logger.error("update failed")
        return False

    return True


def check_submodule(path):
    if not os.path.exists(path):
        return True
    branches = git_branch_status(path, options="-a --contains HEAD")
    return len(branches) > 0

def update_submodule(path, fetch_url, params):
    push_url = params.get("push", None)
    ok = True

    if (os.path.exists(path) and
        not oebakery.call("git submodule sync -- %s"%(path))):
        logger.error("Failed to synchronize git submodule")

    status = git_submodule_status(path)
    if status and status[0] == "-":
        cmd = "git submodule update --init --recursive"
        if not oebakery.call("%s %s"%(cmd, path)):
            logger.error("Failed to add submodule: %s", path)
            return False

    elif status is None:
        cmd = "git submodule add -f"
        if "branch" in params:
            branch = params["branch"]
            if branch != "master":
                cmd += " -b %s"%(params["branch"])
        if not oebakery.call("%s -- %s %s"%(cmd, fetch_url, path)):
            logger.error("Failed to add submodule: %s", path)
            return False
        if not oebakery.call(
            "git submodule update --init --recursive %s"%(path)):
            logger.error("Failed to initialize submodule: %s", path)
            return False

    # set push_default to 'tracking' if unset
    push_default = oebakery.call('git config --get push.default',
                                 dir=path, quiet=True)
    if (push_default == None):
        oebakery.call('git config push.default tracking', dir=path)

    # update origin fetch url if necessary
    current_url = oebakery.call('git config --get remote.origin.url',
                                dir=path, quiet=True)
    if current_url:
        current_url = current_url.strip()
    if fetch_url != current_url:
        if not oebakery.call('git config remote.origin.url %s'%(fetch_url),
                             dir=path):
            logger.error("Failed to set origin url %s for %s", fetch_url, path)
            ok = False

    # set push url as specified
    current_url = oebakery.call('git config --get remote.origin.pushurl',
                                dir=path, quiet=True)
    if current_url:
        current_url = current_url.strip()
    if push_url and current_url != push_url:
        if not oebakery.call('git config remote.origin.pushurl %s'%(push_url),
                             dir=path):
            logger.error("Failed to set origin push url %s for %s",
                         push_url, path)
            ok = False

    if not oebakery.call('git remote update --prune origin', dir=path):
        logger.error("failed to update remote origin")
        ok = False

    # remove override of push url if not specified
    if not push_url and current_url:
        if not oebakery.call('git config --unset remote.origin.pushurl',
                             dir=path):
            logger.error("Failed to unset origin push url for %s", path)
            ok = False

    # update remotes
    for (name, url) in params.get("remote", []):
        if not git_update_remote(name, url, path=path):
            logger.error("update of remote %s failed", name)
            ok = False

    #
    if "tag" in params:
        tag = params["tag"]
        if not oebakery.call("git checkout --detach refs/tags/%s"%(tag),
                             dir=path):
            logger.error("Failed to checkout submodule %s tag %s",
                         path, tag)
            return False
    elif "commit" in params:
        commit = params["commit"]
        if not oebakery.call("git checkout --detach %s"%(commit), dir=path):
            logger.error("Failed to checkout submodule %s commit %s",
                         path, commit)
            return False
    elif "branch" in params:
        branch = params["branch"]
        branches = git_branch_status(path)
        if branch in branches:
            if not branches[branch]["current"]:
                if not oebakery.call("git checkout %s"%(branch), dir=path):
                    logger.error("Failed to checkout submodule %s branch %s",
                                 path, branch)
                    return False
        else:
            if not oebakery.call("git checkout -t -b %s %s"%(
                    branch, "origin/%s"%(branch)), dir=path):
                logger.error("Failed to checkout%s branch %s", name, branch)
                return False
        current_remote = oebakery.call(
            "git config --get branch.%s.remote"%(branch),
            dir=path, quiet=True)
        current_merge = oebakery.call(
            "git config --get branch.%s.merge"%(branch),
            dir=path, quiet=True)
        if (current_remote is  None or current_merge is None or
            current_remote.strip() != "origin" or
            current_merge.strip() != "refs/heads/%s"%(branch)):
            if not oebakery.call(
                "git branch --set-upstream %s origin/%s"%(branch, branch),
                dir=path):
                ok = False
    else:
        if not oebakery.call(
            "git submodule update --init --recursive %s"%(path)):
            logger.error("Failed to initialize submodule: %s", path)
            ok = False

    return ok


def git_branch_status(path=None, options=None, nobranch=False):
    cmd = "git branch -v --no-abbrev"
    if options:
        cmd += " " + options
    branches = {}
    for line in oebakery.call(cmd, quiet=True, dir=path).split("\n"):
        if not line:
            continue
        asterix = line[0]
        if asterix == "*":
            current = True
        else:
            current = False
        line = line[2:]
        if line[:11] == "(no branch)":
            if not nobranch:
                continue
            name = ""
            line = line[11:]
            line.lstrip()
        else:
            (name, line) = line.split(None, 1)
        commitid, subject = line.split(None, 1)
        branches[name] = {
            "current"	: current,
            "commitid"	:commitid,
            "subject"	: subject,
            }
    return branches


def git_submodule_status(path):
    status = oebakery.call('git submodule status', quiet=True)
    if not status:
        return None
    for line in status.split('\n'):
        if len(line) == 0:
            continue
        if (path == line[1:].split()[1]):
            prefix = line[0]
            commitid = line[1:].split()[0]
            return (prefix, commitid)
    return None


def git_update_remote(name, url, path=None):
    ok = True

    url_split = url.split(",")
    if len(url_split) < 1 or len(url_split) > 2:
        logger.error("Invalid remote url: %s", url)
        return False
    if len(url_split) == 2:
        push_url = url_split[1]
    else:
        push_url = None
    fetch_url=url_split[0]

    # get list of remotes and their urls
    remotes_list = oebakery.call('git remote -v', dir=path, quiet=True)
    if remotes_list == None:
        logger.error("Failed to get list of remotes")
        return False
    else:
        remotes_list = remotes_list.split('\n')

    # get fetch urls
    remotes_fetch = {}
    for remote in remotes_list:
        if '\t' in remote:
            (iter_name, iter_url) = string.split(remote, '\t', maxsplit=1)
            if iter_url[-8:] == ' (fetch)':
                remotes_fetch[iter_name] = iter_url[:-8]
            elif iter_url[-7:] == ' (push)':
                # don't take push url here, but use git config to know
                # if it is actually set, or just derived from fetch url
                pass
            else:
                # pre git-1.6.4 did only have fetch url's
                remotes_fetch[iter_name] = iter_url

    # get push urls
    remotes_push = {}
    for iter_name in remotes_fetch.keys():
        iter_url = oebakery.call('git config --get remote.%s.pushurl'%iter_name,
                                 dir=path, quiet=True)
        if iter_url:
            remotes_push[iter_name] = iter_url.strip()

    # if remote is not found, add it and return
    if not remotes_fetch.has_key(name):
        if not oebakery.call('git remote add %s %s'%(name, fetch_url),
                             dir=path):
            logger.error("Failed to add remote %s", name)
            return False
        # also add push url if specified
        if push_url:
            if not oebakery.call(
                'git config remote.%s.pushurl %s'%(name, push_url),
                                 dir=path):
                logger.error("failed to set pushurl %s for %s", push_url, name)
                ok = False

        return ok

    # change (fetch) url if not matching
    if remotes_fetch[name] != fetch_url:
        if not oebakery.call('git config remote.%s.url %s'%(name, fetch_url),
                             dir=path):
            logger.error("failed to set url %s for %s", name, fetch_url)
            ok = False

    # if push url is different url, change it
    if push_url:
        if not remotes_push.has_key(name) or remotes_push[name] != push_url:
            if not oebakery.call(
                'git config remote.%s.pushurl %s'%(name, push_url),
                dir=path):
                logger.error("failed to set pushurl %s for %s", push_url, name)
                ok = False

    # if push url is currently set, but shouldn't be, remove it
    else:
        if remotes_push.has_key(name) and remotes_push[name] != fetch_url:
            if not oebakery.call('git config --unset remote.%s.pushurl'%(name),
                                 dir=path):
                logger.error("failed to unset pushurl for %s", name)
                ok = False

    return ok



from __future__ import with_statement # This isn't required in Python 2.6

__version__ = "4.0.2"

__all__ = []

import log
from log import logger, fatal
__all__ += [ "log", "logger", "fatal" ]

import cmd
__all__ += [ "cmd" ]


import data
__all__ += [ "data" ]

import parse
__all__ += [ "parse" ]

import path
from path import set_topdir, get_topdir, chdir
__all__ += [ "set_topdir", "get_topdir", "chdir" ]

import shell
from oebakery.shell import call
__all__ += [ "shell", "call" ]


import os
oldpwd = os.getcwd()
__all__ += [ "oldpwd" ]

#
# legacy API - backwards compatibility with OE-lite Bakery 3.x
#

from log import debug, info, warn, err, die

class FatalError(Exception):
    pass

# Logging/output API for manifest commands:
#
# print - for console
#
# import oebakery
#
# oebakery.fatal()
#
# import logging
# logging.debug()
# logging.info()
# logging.warning() - when client/recipe is not responsible for the warning
# logging.error()
#
# import warnings
# warnings.warn() - when client/recipe should be fixed to eliminate the warning
#
# Legacy API (deprecated):
#
# oebakery.debug()
# oebakery.info()
# oebakery.warn()
# oebakery.err()
# oebakery.die()
#
# oebakery.FatalError() - Exception class


#
# New style meta:
#  + has own manifest commands (in lib/oelite/cmd/)
#  + has oelite.baker.add_bake_parser_options(parser) function
#  + has oelite.baker.add_show_parser_options(parser) function
#  + has oelite.baker.OEliteBaker().bake() method
#  + has oelite.baker.OEliteBaker().show() method
#  + configures logging module by itself (to work with bakery 3)
#  + does not use oebakery.DEBUG
#  + does not use oebakery.FatalError()
#  + does not use deprecated oebakery.debug(), oebakery.info() ....
#  + uses logging module
#  + does not use oebakery.die()
#  + uses oebakery.fatal()
#
# Old style meta
#  + does not have own manifest commands (in lib/oelite/cmd/)
#  + has oelite.baker.add_bake_parser_options(parser) function
#  + has oelite.baker.add_show_parser_options(parser) function
#  + has oelite.baker.OEliteBaker().bake() method
#  + has oelite.baker.OEliteBaker().show() method
#  + does normally not use the logging module
#  + uses oebakery.DEBUG
#  + uses oebakery.FatalError()
#  + uses oebakery.debug(), oebakery.info() ....
#  + uses oeabkery.die()



from __future__ import with_statement # This isn't required in Python 2.6

from cmds import \
    clear, \
    add_builtin_cmds, \
    add_manifest_cmds, \
    cmds_usage, \
    get_cmd, \
    cmd_parser, \
    call

__all__ = [
    "clear",
    "add_builtin_cmds",
    "add_manifest_cmds",
    "cmds_usage",
    "get_cmd",
    "cmd_parser",
    "call",
]

builtin_cmds = [ "init", "clone", "update", "pull" ]
__all__ += builtin_cmds



import os

import oebakery
logger = oebakery.logger
import update


# oe pull
# pull oe-lite.git repo and all submodules, branch submodules are git
# pull'ed, non-branch submodules are git submodule add'ed.

# oe pull . meta/core meta/base
# pull oe-lite.git repo and submodules meta/core and meta/base.

# oe pull meta/core
# pull submodule meta/core.


args = "[<path>...]"
description = ("Fetch changes from upstream repositories", """
  When called without path arguments, fetch and merge head of main repository,
  checkout the submodule versions stored in main repository for all non-branch
  submodules, and fetch and merge head of branch submodules.

  When called with a list of submodule paths, pull only those modules.
  Use "." to specify main repository.""")


def add_parser_options(parser):
    parser.add_option("-r", "--remotes",
                      action="store_true", dest="remotes", default=False,
                      help="fetch updates for remote tracking branches")
    return


def run(options, args, config):
    if not args or "." in args:
        options.main = True
    else:
        options.main = False
    paths = []
    submodules = []
    for path, url, params in config["__submodules"]:
        if not args or path in args:
            paths.append(path)
            submodules.append((path, url, params))

    if not os.path.exists(".git"):
        return "Aiee!  This is not even a git repository: %s"%(os.getcwd())

    if options.main:
        if options.remotes and not git_remote_update():
            return 1
        if not git_pull():
            return 1

    if options.remotes:
        for path in paths:
            git_remote_update(path)

    # get list of submodules
    #current = {}
    #for line in oebakery.call("git submodule status", quiet=True)\
    #        .split("\n"):
    #    if len(line) == 0:
    #        continue
    #
    #    path = line[1:].split()[1]
    #    prefix = line[0]
    #    commitid = line[1:].split()[0]
    #
    #    fetch_url = None
    #    push_url = None
    #    branch = None
    #    remotes = None
    #
    #    current[path] = (prefix, commitid)

    ok = True
    for path, url, params in submodules:
        if not update.update_submodules([(path, url, params)]):
            ok = False
            continue
        if "branch" in params:
            if not git_pull(path, params["branch"]):
                ok = False

    return not ok


def git_pull(path=None, branch=None):
    if path:
        name = " submodule %s"%(path)
    else:
        name = ""
    if not oebakery.call("git pull", dir=path):
        logger.error("Failed to pull %s"%(name))
        return False
    return True


def git_remote_update(path=None):
    # older git versions return 1 on success, and newer versions 0
    # so we will just ignore the return code for now
    oebakery.call("git remote update", dir=path)
    remotes = oebakery.call("git remote", dir=path, quiet=True)
    ok = True
    if remotes:
        for remote in remotes.split("\n"):
            if not remote or len(remote) == 0:
                continue
            if not oebakery.call("git remote prune %s"%(remote),
                                 dir=path):
                msg = "Failed to prune remote %s"%(remote)
                if path:
                    msg += " in %s"%(path)
                logger.error(msg)
                ok = False
    return ok



import logging
import sys
import oebakery


class ConsoleFormatter(logging.Formatter):

    def __init__(self):
        logging.Formatter.__init__(self)
        return

    def format(self, record):
        record.message = record.getMessage()
        fmt = ""
        if not record.levelno == logging.INFO:
            fmt += "%(levelname)s: "
        if record.levelno == logging.DEBUG:
            fmt += "%(name)s: "
        fmt += "%(message)s"
        s = fmt % record.__dict__
        if record.exc_info:
            if not record.exc_text:
                record.exc_text = self.formatException(record.exc_info)
        if record.exc_text:
            if s[-1:] != "\n":
                s += "\n\n"
            s = s + record.exc_text + "\n"
        return s

logging.basicConfig(format="%(message)s")

logger = logging.getLogger("bakery")

console_logger = logging.StreamHandler()
console_logger.setLevel(logging.DEBUG)
console_formatter = ConsoleFormatter()
console_logger.setFormatter(console_formatter)

logger.addHandler(console_logger)
logger.propagate = False

def fatal(msg):
    logging.critical("FATAL: %s", msg)
    raise oebakery.FatalError(msg)


# Legacy API (deprecated, for compatibility with old meta-data)

def legacy_warn(msg):
    logging.warning("WARNING: %s", msg)

def legacy_err(msg):
    logging.error("ERROR: %s", msg)

def configure_legacy_logging(debug):
    if debug:
        oebakery.DEBUG = True
        logging.getLogger().setLevel(logging.DEBUG)
    else:
        oebakery.DEBUG = False
        logging.getLogger().setLevel(logging.INFO)
    return

debug = logging.debug
info = logging.info
warn = legacy_warn
err = legacy_err
die = fatal



import sys
import os
import ply.lex, ply.yacc

import oebakery
logger = oebakery.logger


class ParseError(Exception):

    def __init__(self, parser, msg, details=None, symbol=None, lineno=None):
        self.parser = parser
        self.msg = msg
        self.details = details
        if lineno is not None:
            self.errlineno = lineno
        elif "lexer" in dir(self.details):
            self.errlineno = self.details.lexer.lineno
        else:
            self.errlineno = self.details.lineno
        if symbol:
            self.symbol = symbol
        #elif self.details is None:
        #    self.symbol = ""
        elif not "value" in dir(self.details):
            self.symbol = None
        elif self.details.type == "error":
            self.symbol = self.details.value[0]
        else:
            self.symbol = self.details.value
        try:
            self.lexer = self.details.lexer
        except:
            self.lexer = self.parser.lexer
        if not self.symbol:
            self.msg += " in %s at line %d"%(
                parser.filename or "<unknown file>",
                self.lexer.lineno + 1)
        elif self.details.type == "error":
            self.msg += " in %s at line %d: %s"%(
                parser.filename or "<unknown file>",
                self.lexer.lineno + 1,
                repr(self.symbol))
        else:
            self.msg += " in %s at line %d: %s %s"%(
                parser.filename or "<unknown file>",
                self.lexer.lineno + 1,
                self.details.type, repr(self.symbol))
        return

    def __str__(self):
        return self.msg

    def print_details(self):
        print self.msg
        if not self.details:
            return ""
        lines = self.parser.text.splitlines()
        #errlineno = self.details.lineno
        #errlineno = self.lexer.lineno
        firstline = max(self.errlineno - 5, 0)
        lastline = min(self.errlineno + 5, len(lines))
        errlinepos = 0
        for lineno in xrange(self.errlineno):
            errlinepos += len(lines[lineno]) + 1
        #if isinstance(self.details, ply.lex.LexToken):
        #    print "this is a LexToken"
        #    lexpos = self.details.lexpos
        #elif isinstance(self.details, ply.lex.Lexer):
        #    print "this is a Lexer"
        #    lexpos = self.details.lexpos - len(self.symbol)
        try:
            lexpos = self.details.lexpos
            errpos = (lexpos - 1) - errlinepos
            errlinebefore = lines[self.errlineno]\
                [:(lexpos - errlinepos)]
            errlinebefore = len(errlinebefore.expandtabs())
        except AttributeError:
            lexpos = None
        linenofmtlen = len(str(lastline))
        lineprintfmt = "%%s%%%dd %%s"%(linenofmtlen)
        for lineno in xrange(firstline, lastline):
            if lineno == self.errlineno:
                if lineno:
                    print ""
                prefix = "-> "
            else:
                prefix = "   "
            print lineprintfmt%(prefix, lineno + 1,
                                lines[lineno].expandtabs())
            if lineno == self.errlineno:
                if lexpos:
                    prefixlen = len(prefix) + linenofmtlen + 1
                    print "%s%s"%(" "*(prefixlen + errlinebefore),
                                  "^"*len(self.symbol))
                else:
                    print ""
        if self.parser.parent:
            parent = self.parser.parent
            print "Included from %s"%(parent.filename)
            parent = parent.parent
            while parent:
                print "              %s"%(parent.filename)
        return


class BakeryParser(object):

    def __init__(self, data=None):
        self.lexer = ply.lex.lex(module=self)
        tabmodule = self.__class__.__module__ + "_tab"
        self.yacc = ply.yacc.yacc(module=self, debug=0, write_tables=0)
        if data is not None:
            self.data = data
        else:
            self.data = oebakery.data.BakeryData()
        return


    reserved = {
        'export'	: 'EXPORT',
        'require'	: 'REQUIRE',
        'include'	: 'INCLUDE',
        'inherit'	: 'INHERIT',
        'fakeroot'	: 'FAKEROOT',
        'python'	: 'PYTHON',
        'addtask'	: 'ADDTASK',
        'addhook'	: 'ADDHOOK',
        'def'		: 'DEF',
        'del'		: 'DEL',
        'freeze'	: 'FREEZE',
        'unfreeze'	: 'UNFREEZE',
        }

    tokens = [
        'VARNAME', 'FLAG',
        'ASSIGN', 'EXPASSIGN', 'WEAKASSIGN', 'LAZYASSIGN',
        'APPEND', 'PREPEND', 'POSTDOT', 'PREDOT',
        'STRING',
        'NEWLINE', 'COMMENT',
        ]

    t_ignore = ' \t'


    def t_VARNAME(self, t):
        r'[a-zA-Z_][a-zA-Z0-9_\-\${}/\+\.]*'
        if self.reserved.get(t.value):
            raise ParseError(
                self, "Reserved word not allowed in OE-lite Bakery configuration",
                t, t.value)
        return t

    def t_FLAG(self, t):
        r'\[[a-zA-Z_][a-zA-Z0-9_]*\]'
        t.value = t.value[1:-1]
        return t


    def t_APPEND(self, t):
        r'\+='
        return t

    def t_PREDOT(self, t):
        r'\.='
        return t

    def t_LAZYASSIGN(self, t):
        r'\?\?='
        return t

    def t_WEAKASSIGN(self, t):
        r'\?='
        return t

    def t_EXPASSIGN(self, t):
        r':='
        return t

    def t_PREPEND(self, t):
        r'=\+'
        return t

    def t_POSTDOT(self, t):
        r'=\.'
        return t

    def t_ASSIGN(self, t):
        r'='
        return t


    def t_COMMENT(self, t):
        r'\#[^\n]*'
        pass # no return value, token discarded

    def t_NEWLINE(self, t):
        r'\n+'
        t.lexer.lineno += len(t.value)
        return t


    def t_EMPTYSTRING(self, t):
        r'""|\'\''
        t.type = "STRING"
        t.value = ""
        return t

    def t_DQUOTESTRING(self, t):
        r'"(\\"|\\\n|[^"\n])*?"'
        t.type = "STRING"
        newlines = t.value.count('\n')
        if newlines:
            t.lineno += newlines
        t.value = t.value[1:-1].replace('\\\n','').replace('\\"','"')
        return t

    def t_SQUOTESTRING(self, t):
        r"'(\\'|\\\n|[^'\n])*?'"
        t.type = "STRING"
        t.lineno += t.value.count('\n')
        t.value = t.value[1:-1].replace('\\\n','').replace("\\'","'")
        return t


    def t_UNTERMINATEDDQUOTESTRING(self, t):
        r'"(\\"|\\\n|[^"\n])*?\n'
        t.lineno += t.value.count('\n')
        raise ParseError(self, "Unterminated string", t, t.value)

    def t_UNTERMINATEDSQUOTESTRING(self, t):
        r"'(\\'|\\\n|[^'\n])*?\n"
        t.lineno += t.value.count('\n')
        raise ParseError(self, "Unterminated string", t, t.value)


    start = 'syntax'

    def p_syntax(self, p):
        '''syntax : statement
                  | statement syntax'''
        return

    def p_statement(self, p):
        '''statement : NEWLINE
                     | assignment NEWLINE
                     | COMMENT'''
        return

    def p_variable(self, p):
        '''variable : VARNAME'''
        p[0] = (p[1], '')
        return

    def p_flag(self, p):
        '''flag : VARNAME FLAG'''
        p[0] = (p[1], p[2])
        return

    def p_varflag(self, p):
        '''varflag : flag
                   | variable'''
        p[0] = p[1]
        return

    def p_simple_assignment(self, p):
        '''assignment : varflag ASSIGN STRING'''
        self.data.set_flag(p[1][0], p[1][1], p[3])
        return

    def p_exp_assignment(self, p):
        '''assignment : varflag EXPASSIGN STRING'''
        print "EXPASSIGN %s"%(p[1][0])
        self.data.set_flag(p[1][0], p[1][1], self.data.expand(p[3], 1))
        return

    def p_defaultval_assignment(self, p):
        '''assignment : variable LAZYASSIGN STRING'''
        self.data.set_flag(p[1][0], "defaultval", p[3])
        return

    def p_weak_assignment(self, p):
        '''assignment : varflag WEAKASSIGN STRING'''
        if self.data.get_flag(p[1][0], p[1][1]) == None:
            self.data.set_flag(p[1][0], p[1][1], p[3])
        return

    def p_append_assignment(self, p):
        '''assignment : varflag APPEND STRING'''
        self.data.append_flag(p[1][0], p[1][1], p[3])
        return

    def p_prepend_assignment(self, p):
        '''assignment : varflag PREPEND STRING'''
        self.data.prepend_flag(p[1][0], p[1][1], p[3])
        return

    def p_predot_assignment(self, p):
        '''assignment : varflag PREDOT STRING'''
        self.data.append_flag(p[1][0], p[1][1], p[3], separator="")
        return

    def p_postdot_assignment(self, p):
        '''assignment : varflag POSTDOT STRING'''
        self.data.prepend_flag(p[1][0], p[1][1], p[3], separator="")
        return

    def t_ANY_error(self, t):
        raise ParseError(self, "Illegal character in %s at line %d: %s"%(
                self.filename or "<unknown>", t.lineno+1, repr(t.value[0])),
                         t, t.value)

    def p_error(self, p):
        raise ParseError(
            self, "Syntax error in %s at line %d: type=%s value=%s"%(
                self.filename or "<unknown>", p.lineno+1, p.type, p.value),
            p, p.value)

    def parse(self, filename, require=True):
        self.filename = os.path.realpath(filename)

        if not os.path.exists(self.filename):
            print "No such file: %s"%(filename)
            return None

        mtime = os.path.getmtime(self.filename)
        f = open(self.filename)
        self.text = f.read()
        f.close()
        self.data.set_file_mtime(self.filename, mtime)

        self.lexer.lineno = 0
        self.yacc.parse(self.text + '\n', lexer=self.lexer)

        return self.data


#class ParseError(Exception):
#
#    def __init__(self, parser, msg, details=None, symbol=None):
#        self.parser = parser
#        self.msg = msg
#        self.details = details
#        if not symbol and details:
#            self.symbol = details.value
#        else:
#            self.symbol = symbol
#        return
#
#    def __str__(self):
#        return self.msg
#
#    def print_details(self):
#        print self.msg
#        if not self.details:
#            return ""
#        lines = self.parser.text.splitlines()
#        firstline = max(self.details.lineno - 5, 0)
#        lastline = min(self.details.lineno + 5, len(lines) - 1)
#        errlinepos = 0
#        for lineno in xrange(self.details.lineno):
#            errlinepos += len(lines[lineno]) + 1
#        errpos = (self.details.lexpos - 1) - errlinepos
#        errlinebefore = lines[self.details.lineno]\
#            [:(self.details.lexpos - errlinepos)]
#        errlinebefore = len(errlinebefore.expandtabs())
#        linenofmtlen = len(str(lastline))
#        lineprintfmt = "%%s%%%dd %%s"%(linenofmtlen)
#        for lineno in xrange(firstline, lastline):
#            if lineno == self.details.lineno:
#                if lineno:
#                    print ""
#                prefix = "-> "
#            else:
#                prefix = "   "
#            print lineprintfmt%(prefix, lineno + 1,
#                                lines[lineno].expandtabs())
#            if lineno == self.details.lineno:
#                prefixlen = len(prefix) + linenofmtlen + 1
#                print "%s%s"%(" "*(prefixlen + errlinebefore),
#                              "^"*len(self.symbol))
#        return


def parse(path):
    confparser = BakeryParser()
    return confparser.parse(path)


def parse_bakery_conf():
    topdir = oebakery.get_topdir()
    oebakery.chdir(topdir)

    config = parse("conf/bakery.conf")
    if config is None:
        return None

    ok = True

    OEPATH = []
    OERECIPES = []
    PYTHONPATH = []

    OESTACK = config.get("OESTACK") or ""
    config["__oestack"] = []
    config["__submodules"] = []
    for oestack in OESTACK.split():
        oestack = oestack.split(";")
        path = oestack[0]
        params = {}
        for param in oestack[1:]:
            key, value = param.split("=", 1)
            if key == "remote":
                if not key in params:
                    params[key] = []
                name, url = value.split(",", 1)
                params[key].append((name, url))
            else:
                params[key] = value
        if path.startswith("meta/"):
            if not "oepath" in params:
                params["oepath"] = ""
            if not "pythonpath" in params:
                params["pythonpath"] = "lib"
            if not "oerecipes" in params:
                params["oerecipes"] = "*/*.oe"
        if "srcuri" in params and params["srcuri"].startswith("git://"):
            if not "protocol" in params:
                params["protocol"] = "git"
            url = "%s%s"%(params["protocol"], params["srcuri"][3:])
            config["__submodules"].append((path, url, params))
        config["__oestack"].append((path, params))
        if "oepath" in params:
            if params["oepath"] == "":
                oepath = path
            else:
                oepath = os.path.join(path, params["oepath"])
            OEPATH.append(oepath)
            oerecipes_base = os.path.join(oepath, "recipes")
            if os.path.isdir(oerecipes_base):
                for oerecipes in params["oerecipes"].split(":"):
                    OERECIPES.append(os.path.join(oerecipes_base, oerecipes))
        if "pythonpath" in params:
            if params["pythonpath"] == "":
                PYTHONPATH.append(os.path.abspath(path))
            else:
                pythonpath = os.path.join(path, params["pythonpath"])
                if os.path.exists(pythonpath):
                    PYTHONPATH.append(os.path.abspath(pythonpath))

    OEPATH.insert(0, ".")
    if os.path.isdir("recipes"):
        OERECIPES.insert(0, "recipes/*/*.oe")

    config["OEPATH"] = ":".join(map(os.path.abspath, OEPATH))
    config["OEPATH_PRETTY"] = ":".join(OEPATH)

    config["OERECIPES"] = ":".join(map(os.path.abspath, OERECIPES))
    config["OERECIPES_PRETTY"] = ":".join(OERECIPES)

    config["TOPDIR"] = topdir

    sys.path = PYTHONPATH + sys.path

    logger.debug("OEPATH = %s", config["OEPATH_PRETTY"])
    logger.debug("OERECIPES = %s", config["OERECIPES_PRETTY"])
    logger.debug("PYTHONPATH = %s", PYTHONPATH)

    return config



#! /usr/bin/env python

#import sys
#import os
#sys.path.append(os.path.join(os.path.dirname(
#            os.path.abspath(sys.argv[0])),
#            "/setuptools/setuptools-0.6c11-py2.6.egg"))
#from ez_setup import use_setuptools
#use_setuptools()

import oebakery
version = oebakery.__version__

import sys
if "--version" in sys.argv:
    print version
    sys.exit(0)

from setuptools import setup, find_packages

try:
    import stdeb
except ImportError:
    print 'Warning: cannot find stdeb module'

setup(name='oe-lite',
      version=version,
      description="OE-lite - embedded Linux platform development kit",
      long_description="""\
A tool for building complete embedded Linux platforms""",
      classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='',
      author="Esben Haabendal",
      author_email="esben.haabendal@prevas.dk",
      license="GPL",
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      #packages=['oebakery'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
      ],
      url="http://oe-lite.org/",
      entry_points = {
        'console_scripts': [
            'oe = oebakery.oe:main',
            ]
        }
      )



import sys
import os
import shutil

from oebakery import *


__topdir = None


def set_topdir(dir):
    global __topdir
    if not (os.path.exists(os.path.join(dir, "conf", "bakery.conf"))):
        logger.critical("Not a valid OE-lite manifest: %s"%dir)
    __topdir = os.path.abspath(dir)
    return __topdir


def get_topdir():
    global __topdir
    if __topdir is None:
        __topdir = locate_topdir()
    return __topdir


def locate_topdir():
    if (os.path.exists(os.path.join(os.getcwd(), 'conf', 'bakery.conf'))):
        # PWD might not be set correctly, so we have to try
        # os.getcwd() first, which is ok as long as we don't recurse
        # into it.  This was experienced when running under Buildbot.
        topdir = os.getcwd()
    else:
        topdir = locate_topdir_recursive(os.getenv('PWD'))
    if not topdir:
        logger.critical("current directory is not part of an OE-lite manifest")
        sys.exit(1)
    return os.path.realpath(topdir)


def locate_topdir_recursive(dir):
    if dir == '/':
        return None
    if (os.path.exists(os.path.join(dir, 'conf', 'bakery.conf'))):
        return os.path.abspath(dir)
    return locate_topdir_recursive(os.path.dirname(dir))


def chdir(dir, quiet=False):
    logger.debug("chdir: %s", dir)
    try:
        cwd = os.getcwd()
    except OSError, e:
        cwd = None
    if (cwd and
        os.path.realpath(os.path.normpath(dir)) == os.path.normpath(cwd)):
        return

    if not quiet:
        print '> cd', dir

    os.chdir(dir)

    return


def copy_local_conf_sample(confdir):
    config = os.path.join(confdir, 'local.conf')
    sample = os.path.join(confdir, 'local.conf.sample')
    if not os.path.exists(config) and os.path.exists(sample):
        print '> cp %s %s'%(sample, config)
        try:
            shutil.copyfile(sample, config)
        except Exception, e:
            logger.warning("failed to write config file: %s: %s", config, e)



import oebakery
import oelite.baker


description = "Build something"


def add_parser_options(parser):
    oelite.baker.add_bake_parser_options(parser)
    parser.add_option("-d", "--debug",
                      action="store_true", default=False,
                      help="Debug the OE-lite metadata")
    return


def run(options, args, config):
    oebakery.log.configure_legacy_logging(options.debug)
    baker = oelite.baker.OEliteBaker(options, args, config)
    return baker.bake()



import sys
import os
import subprocess

from oebakery import *


def call(cmd, dir=None, quiet=False, success_returncode=0):
    logger.debug("call: %s", cmd)

    if type(cmd) == type([]):
        cmdlist = cmd
        cmd = cmdlist[0]
        for arg in cmdlist[1:]:
            cmd = cmd + ' ' + arg

    if dir:
        if not os.path.exists(dir):
            return None
    else:
        dir = ""

    # Git seems to have a problem with long paths __and__ generates
    # long relative paths when run from a symlinked path.  To
    # workaround this, we always switch to realpath before running any
    # commands.
    rdir = os.path.realpath(dir)

    pwd = os.getcwd()
    chdir(rdir, quiet=True)

    if not quiet:
        if dir:
            print '%s> %s'%(dir, cmd)
        else:
            print '> %s'%(cmd)

    retval = None
    if quiet:
        process = subprocess.Popen(cmd, shell=True, stdin=sys.stdin,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.STDOUT)
        output = process.communicate()[0]
        if process.returncode == success_returncode:
            retval = output

    else:
        returncode = subprocess.call(cmd, shell=True, stdin=sys.stdin)
        if returncode == success_returncode:
            retval = True

    chdir(pwd, quiet=True)

    return retval



import os

import oebakery


description = "Create new OE-lite manifest repository"
arguments = None
flags = ("no-oestack")


# To be run from a directory that is _not_ a git repository yet.

def run(options, args, config):
    topdir = oebakery.set_topdir(os.path.curdir)

    if oebakery.call("git status", quiet=True):
        oebakery.fatal("Already initialized: %s"%(topdir))

    if not oebakery.call("git init"):
        oebakery.fatal("Failed to initialize git")

    if not oebakery.call("git config push.default tracking"):
        oebakery.fatal("Failed to set push.default = tracking")

    oebakery.path.copy_local_conf_sample(config["CONFDIR"] or "conf")

    return ["update"]



import re
import types


NO_EXPANSION      = 0
FULL_EXPANSION    = 1


class ExpansionStack:

    def __init__(self):
        self.stack = []
        self.python = False
        return

    def push(self, var):
        if var in self.stack:
            raise Exception("Circular expansion: %s"%("->".join(self.stack)))
        self.stack.append(var)
        return

    def pop(self):
        del self.stack[-1:]
        return

    def __len__(self):
        return len(self.stack)

    def __str__(self, prefix="  "):
        return prefix + ("\n%s"%(prefix)).join(self.stack)


class BakeryData:

    def __init__(self):
        self._dict = {}
        self.mtime = {}
        self.expand_stack = ExpansionStack()
        return

    def dict(self):
        return self._dict

    def set(self, var, value):
        return self.set_flag(var, "", value)

    def get(self, var, expand=FULL_EXPANSION):
        return self._get(var, expand)[0]

    def _get(self, var, expand=FULL_EXPANSION):
        #print "_get var=%s expand=%s"%(var, expand)
        assert isinstance(expand, int)
        try:
            val = self._dict[var][""]
        except KeyError:
            try:
                val = self._dict[var]["defaultval"]
            except KeyError:
                val = None
        if not expand:
            return (val, None)
        if not var in self._dict:
            return (None, None)
        if not isinstance(val, (basestring, types.NoneType)):
            return (val, None)

        deps = set()
        #print "get expanding %s=%s"%(var, repr(val))
        expand_method = self.get_flag(var, "expand")
        if expand_method:
            expand_method = int(expand_method)
        else:
            expand_method = expand
        if expand_method != NO_EXPANSION and val:
            self.expand_stack.push("${%s}"%var)
            (val, deps) = self._expand(val, expand_method)
            self.expand_stack.pop()

        return (val, deps)

    def set_flag(self, var, flag, value):
        if not var in self._dict:
            self._dict[var] = {}
        self._dict[var][flag] = value
        return

    def get_flag(self, var, flag):
        try:
            return self._dict[var][flag]
        except KeyError:
            return None

    def del_var(self, var):
        try:
            del self._dict[var]
        except KeyError:
            pass
        return

    def del_flag(self, var, flag):
        try:
            del self._dict[var][flag]
        except KeyError:
            pass
        return

    #def append(self, var, value, separator=" "):
    #    print "append var=%s value=%s seperator=%s"%(var, value, separator)
    #    current = self.get(var)
    #    if current == None:
    #        self.set(var, value)
    #    else:
    #        self.set(var, current + separator + value)

    def append_flag(self, var, flag, value, separator=" "):
        current = self.get_flag(var, flag)
        if current == None:
            self.set_flag(var, flag, value)
        else:
            self.set_flag(var, flag, current + separator + value)


    #def prepend(self, var, value, separator=" "):
    #    current = self.get(var)
    #    if current == None:
    #        self.set(var, value)
    #    else:
    #        self.set(var, value + separator + current)

    def prepend_flag(self, var, flag, value, separator=" "):
        current = self.get_flag(var, flag)
        if current == None:
            self.set_flag(var, flag, value)
        else:
            self.set_flag(var, flag, value + separator + current)


    #def set_file_mtime(self, filename, path, mtime):
    def set_file_mtime(self, filename, mtime):
        self.mtime[filename] = mtime

    def get_file_mtime(self, filename):
        if not filename in self.mtime:
            return None
        return self.mtime[filename]


    def __repr__(self):
        return '%s()'%(self.__class__.__name__)

    def __str__(self):
        return repr(self._dict)

    def __eq__(self):
        raise Exception("BakeryData.__eq__() not implemented")

    def __hash__(self):
        raise Exception("BakeryData.__hash__() not implemented")

    def __nonzero__(self):
        return len(self._dict) > 0

    def __len__(self): # required by Sized
        return len(self._dict)

    def __getitem__(self, key): # required by Mapping
        return self.get(key, 0)

    def __setitem__(self, key, value): # required by MutableMapping
        self.set(key, value)
        return value

    def __delitem__(self, key): # required by MutableMapping
        self.del_var(key)
        return

    def __iter__(self):
        return self._dict.__iter__

    def __reversed__(self):
        raise Exception("BakeryData.__reversed__() not implemented")

    def __contains__(self, item):
        val = self.get(item, 0)
        return val is not None


    def expand(self, string, method=FULL_EXPANSION):
        """Expand string using variable data.

        Arguments:
        string -- String to expand
        method -- Expansion method (default: FULL_EXPANSION)

        Expansion methods:
        NO_EXPANSION -- no recursive expansion
        FULL_EXPANSION -- full expansion, all variables must be expanded
        """
        #print "expand method=%s string=%s"%(method, repr(string))
        (new_string, deps) = self._expand(string, method)
        return new_string

    def _expand(self, string, method):
        #print "_expand method=%s string=%s"%(method, repr(string))
        orig_string = string
        var_re    = re.compile(r"\${[^@{}]+}")
        python_re = re.compile(r"\${@.+?}")
        deps = set()
        expanded_string = ""
        string_ptr = 0
        for var_match in var_re.finditer(string):
            var = var_match.group(0)[2:-1]
            (val, recdeps) = self._get(var)
            if val is None:
                if method == FULL_EXPANSION:
                    raise Exception("Cannot expand variable ${%s}"%(var),
                                    self.expand_stack)
            expanded_string += (string[string_ptr:var_match.start(0)] +
                                "%s"%(val,))
            string_ptr = var_match.end(0)
            deps.add(var)
            if recdeps:
                deps.union(recdeps)
        expanded_string += string[string_ptr:]
        #print "returning expanded string %s"%(repr(expanded_string))
        return (expanded_string, deps)
